<?php

use TOT\User;

add_shortcode('tot-reputation-status', 'tot_reputation_status_shortcode');
add_shortcode('tot_reputation_status', 'tot_reputation_status_shortcode');

/**
 * Provides short contextual text snippets to describe the current state of the user.
 * @param $attrs
 * @param null $content
 * @return string
 */
function tot_reputation_status_shortcode($attrs, $content = null)
{
    global $tot_plugin_text_domain;

    // normalize attribute keys, lowercase
    if (!empty($attrs)) {
        $attrs = array_change_key_case($attrs, CASE_LOWER);
    }

    $settings = shortcode_atts(
        array(
            'wp-userid' => get_current_user_id(),
            'tot-transaction-id' => '',
            'show-get-verified-button' => 'false'
        ),
        $attrs
    );

    $wpUserid = $settings['wp-userid'];
    if ($wpUserid == 0) {
        return apply_filters('tot_verification_gates_request_signin_block', "<p>You will need to sign-in before you can proceed.</p>");
    }

    if (!isset($settings['app-userid'])) {
        $settings['app-userid'] = tot_user_id($wpUserid);
    }

    $tot_user = new User($wpUserid);
    $reasons = $tot_user->get_reputation_reasons();

    if (is_wp_error($reasons)) {
        return apply_filters('tot_verification_gates_error_block', "<p>There was a problem verifying you - please check back later.</p>");
    }

    $hasReasons = isset($reasons);
    $approved = $hasReasons && $reasons->is_positive('govtIdPositiveReview');
    $pendingReview = $hasReasons && $reasons->is_positive('govtIdPendingReview');
    $rejected = $hasReasons && $reasons->is_negative('govtIdPositiveReview');

    if ($approved) {
        return apply_filters('tot_verification_gates_approved_block', "<p>Thank you - you've been verified!</p>");
    }

    if ($pendingReview) {
        return apply_filters('tot_verification_gates_pending_block', "<p>Thank you - your verification has been submitted! Watch your email inbox. We will get that approved as quickly as we're able to!</p>");
    }

    if ($rejected) {
        return apply_filters('tot_verification_gates_pending_block', "<p>You've not passed verification. Please contact us for more information!</p>");
    }

    $not_verified = !($pendingReview || $rejected || $approved);

    if ($not_verified) {
        $appData = tot_get_user_app_data($wpUserid);
        if (!isset($wpUserid)) {
            $wpUserid = get_current_user_id();
        }
        $appUserid = tot_user_id($wpUserid);
        $verify_person_data = array('person' => $appData, 'appUserid' => $appUserid);
        $verify = new \TOT\API_Person($verify_person_data, 'POST');

        tot_log_as_html_comment('Verifying with', $verify_person_data);
        $error_callback = array($verify, 'handle_verify_person_api_error');
        $verify->verify_result = $verify->send($error_callback);

        return apply_filters("tot_verification_gates_not_verified_block", '<div class="tot-wc-order-validation">'
            . '<a data-tot-verification-required="true" href="#tot_get_verified">'
            . __('Verification', $tot_plugin_text_domain)
            . '</a> '
            . __(' is required before you can proceed.', 'token-of-trust')
            . '</div>');
    }
}
